# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FOCI_RESAMPLE_METADATA = Metadata(
    id="6683a0cb7080f4729048697e248762fc138498eb.workbench",
    name="foci-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_FociResampleLeftSurfacesParamsDictNoTag = typing.TypedDict('_FociResampleLeftSurfacesParamsDictNoTag', {
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleLeftSurfacesParamsDictTagged = typing.TypedDict('FociResampleLeftSurfacesParamsDictTagged', {
    "@type": typing.Literal["left-surfaces"],
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleLeftSurfacesParamsDict = _FociResampleLeftSurfacesParamsDictNoTag | FociResampleLeftSurfacesParamsDictTagged


_FociResampleRightSurfacesParamsDictNoTag = typing.TypedDict('_FociResampleRightSurfacesParamsDictNoTag', {
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleRightSurfacesParamsDictTagged = typing.TypedDict('FociResampleRightSurfacesParamsDictTagged', {
    "@type": typing.Literal["right-surfaces"],
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleRightSurfacesParamsDict = _FociResampleRightSurfacesParamsDictNoTag | FociResampleRightSurfacesParamsDictTagged


_FociResampleCerebellumSurfacesParamsDictNoTag = typing.TypedDict('_FociResampleCerebellumSurfacesParamsDictNoTag', {
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleCerebellumSurfacesParamsDictTagged = typing.TypedDict('FociResampleCerebellumSurfacesParamsDictTagged', {
    "@type": typing.Literal["cerebellum-surfaces"],
    "current-surf": InputPathType,
    "new-surf": InputPathType,
})
FociResampleCerebellumSurfacesParamsDict = _FociResampleCerebellumSurfacesParamsDictNoTag | FociResampleCerebellumSurfacesParamsDictTagged


_FociResampleParamsDictNoTag = typing.TypedDict('_FociResampleParamsDictNoTag', {
    "foci-in": InputPathType,
    "foci-out": str,
    "left-surfaces": typing.NotRequired[FociResampleLeftSurfacesParamsDict | None],
    "right-surfaces": typing.NotRequired[FociResampleRightSurfacesParamsDict | None],
    "cerebellum-surfaces": typing.NotRequired[FociResampleCerebellumSurfacesParamsDict | None],
    "restore-xyz": bool,
    "discard-distance-from-surface": bool,
})
FociResampleParamsDictTagged = typing.TypedDict('FociResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/foci-resample"],
    "foci-in": InputPathType,
    "foci-out": str,
    "left-surfaces": typing.NotRequired[FociResampleLeftSurfacesParamsDict | None],
    "right-surfaces": typing.NotRequired[FociResampleRightSurfacesParamsDict | None],
    "cerebellum-surfaces": typing.NotRequired[FociResampleCerebellumSurfacesParamsDict | None],
    "restore-xyz": bool,
    "discard-distance-from-surface": bool,
})
FociResampleParamsDict = _FociResampleParamsDictNoTag | FociResampleParamsDictTagged


def foci_resample_left_surfaces(
    current_surf: InputPathType,
    new_surf: InputPathType,
) -> FociResampleLeftSurfacesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_surf: the surface the foci are currently projected on.
        new_surf: the surface to project the foci onto.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-surfaces",
        "current-surf": current_surf,
        "new-surf": new_surf,
    }
    return params


def foci_resample_left_surfaces_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FociResampleLeftSurfacesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-surf", None) is None:
        raise StyxValidationError("`current-surf` must not be None")
    if not isinstance(params["current-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-surf` has the wrong type: Received `{type(params.get("current-surf", None))}` expected `InputPathType`')
    if params.get("new-surf", None) is None:
        raise StyxValidationError("`new-surf` must not be None")
    if not isinstance(params["new-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-surf` has the wrong type: Received `{type(params.get("new-surf", None))}` expected `InputPathType`')


def foci_resample_left_surfaces_cargs(
    params: FociResampleLeftSurfacesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-surfaces",
        execution.input_file(params.get("current-surf", None)),
        execution.input_file(params.get("new-surf", None))
    ])
    return cargs


def foci_resample_right_surfaces(
    current_surf: InputPathType,
    new_surf: InputPathType,
) -> FociResampleRightSurfacesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_surf: the surface the foci are currently projected on.
        new_surf: the surface to project the foci onto.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-surfaces",
        "current-surf": current_surf,
        "new-surf": new_surf,
    }
    return params


def foci_resample_right_surfaces_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FociResampleRightSurfacesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-surf", None) is None:
        raise StyxValidationError("`current-surf` must not be None")
    if not isinstance(params["current-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-surf` has the wrong type: Received `{type(params.get("current-surf", None))}` expected `InputPathType`')
    if params.get("new-surf", None) is None:
        raise StyxValidationError("`new-surf` must not be None")
    if not isinstance(params["new-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-surf` has the wrong type: Received `{type(params.get("new-surf", None))}` expected `InputPathType`')


def foci_resample_right_surfaces_cargs(
    params: FociResampleRightSurfacesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-surfaces",
        execution.input_file(params.get("current-surf", None)),
        execution.input_file(params.get("new-surf", None))
    ])
    return cargs


def foci_resample_cerebellum_surfaces(
    current_surf: InputPathType,
    new_surf: InputPathType,
) -> FociResampleCerebellumSurfacesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_surf: the surface the foci are currently projected on.
        new_surf: the surface to project the foci onto.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-surfaces",
        "current-surf": current_surf,
        "new-surf": new_surf,
    }
    return params


def foci_resample_cerebellum_surfaces_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FociResampleCerebellumSurfacesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-surf", None) is None:
        raise StyxValidationError("`current-surf` must not be None")
    if not isinstance(params["current-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-surf` has the wrong type: Received `{type(params.get("current-surf", None))}` expected `InputPathType`')
    if params.get("new-surf", None) is None:
        raise StyxValidationError("`new-surf` must not be None")
    if not isinstance(params["new-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-surf` has the wrong type: Received `{type(params.get("new-surf", None))}` expected `InputPathType`')


def foci_resample_cerebellum_surfaces_cargs(
    params: FociResampleCerebellumSurfacesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-surfaces",
        execution.input_file(params.get("current-surf", None)),
        execution.input_file(params.get("new-surf", None))
    ])
    return cargs


class FociResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FociResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    foci_out: OutputPathType
    """the output foci file"""


def foci_resample_params(
    foci_in: InputPathType,
    foci_out: str,
    left_surfaces: FociResampleLeftSurfacesParamsDict | None = None,
    right_surfaces: FociResampleRightSurfacesParamsDict | None = None,
    cerebellum_surfaces: FociResampleCerebellumSurfacesParamsDict | None = None,
    restore_xyz: bool = False,
    discard_distance_from_surface: bool = False,
) -> FociResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        foci_in: the input foci file.
        foci_out: the output foci file.
        left_surfaces: the left surfaces for resampling.
        right_surfaces: the right surfaces for resampling.
        cerebellum_surfaces: the cerebellum surfaces for resampling.
        restore_xyz: put the original xyz coordinates into the foci, rather\
            than the coordinates obtained from unprojection.
        discard_distance_from_surface: ignore the distance the foci are above\
            or below the current surface.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/foci-resample",
        "foci-in": foci_in,
        "foci-out": foci_out,
        "restore-xyz": restore_xyz,
        "discard-distance-from-surface": discard_distance_from_surface,
    }
    if left_surfaces is not None:
        params["left-surfaces"] = left_surfaces
    if right_surfaces is not None:
        params["right-surfaces"] = right_surfaces
    if cerebellum_surfaces is not None:
        params["cerebellum-surfaces"] = cerebellum_surfaces
    return params


def foci_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FociResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("foci-in", None) is None:
        raise StyxValidationError("`foci-in` must not be None")
    if not isinstance(params["foci-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`foci-in` has the wrong type: Received `{type(params.get("foci-in", None))}` expected `InputPathType`')
    if params.get("foci-out", None) is None:
        raise StyxValidationError("`foci-out` must not be None")
    if not isinstance(params["foci-out"], str):
        raise StyxValidationError(f'`foci-out` has the wrong type: Received `{type(params.get("foci-out", None))}` expected `str`')
    if params.get("left-surfaces", None) is not None:
        foci_resample_left_surfaces_validate(params["left-surfaces"])
    if params.get("right-surfaces", None) is not None:
        foci_resample_right_surfaces_validate(params["right-surfaces"])
    if params.get("cerebellum-surfaces", None) is not None:
        foci_resample_cerebellum_surfaces_validate(params["cerebellum-surfaces"])
    if params.get("restore-xyz", False) is None:
        raise StyxValidationError("`restore-xyz` must not be None")
    if not isinstance(params["restore-xyz"], bool):
        raise StyxValidationError(f'`restore-xyz` has the wrong type: Received `{type(params.get("restore-xyz", False))}` expected `bool`')
    if params.get("discard-distance-from-surface", False) is None:
        raise StyxValidationError("`discard-distance-from-surface` must not be None")
    if not isinstance(params["discard-distance-from-surface"], bool):
        raise StyxValidationError(f'`discard-distance-from-surface` has the wrong type: Received `{type(params.get("discard-distance-from-surface", False))}` expected `bool`')


def foci_resample_cargs(
    params: FociResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-foci-resample"
    ])
    cargs.append(execution.input_file(params.get("foci-in", None)))
    cargs.append(params.get("foci-out", None))
    if params.get("left-surfaces", None) is not None or params.get("right-surfaces", None) is not None or params.get("cerebellum-surfaces", None) is not None:
        cargs.extend([
            *(foci_resample_left_surfaces_cargs(params.get("left-surfaces", None), execution) if (params.get("left-surfaces", None) is not None) else []),
            *(foci_resample_right_surfaces_cargs(params.get("right-surfaces", None), execution) if (params.get("right-surfaces", None) is not None) else []),
            *(foci_resample_cerebellum_surfaces_cargs(params.get("cerebellum-surfaces", None), execution) if (params.get("cerebellum-surfaces", None) is not None) else [])
        ])
    if params.get("restore-xyz", False):
        cargs.append("-restore-xyz")
    if params.get("discard-distance-from-surface", False):
        cargs.append("-discard-distance-from-surface")
    return cargs


def foci_resample_outputs(
    params: FociResampleParamsDict,
    execution: Execution,
) -> FociResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FociResampleOutputs(
        root=execution.output_file("."),
        foci_out=execution.output_file(params.get("foci-out", None)),
    )
    return ret


def foci_resample_execute(
    params: FociResampleParamsDict,
    runner: Runner | None = None,
) -> FociResampleOutputs:
    """
    PROJECT FOCI TO A DIFFERENT SURFACE.
    
    Unprojects foci from the <current-surf> for the structure, then projects
    them to <new-surf>. If the foci have meaningful distances above or below the
    surface, use anatomical surfaces. If the foci should be on the surface, use
    registered spheres and the options -discard-distance-from-surface and
    -restore-xyz.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociResampleOutputs`).
    """
    foci_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOCI_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = foci_resample_cargs(params, execution)
    ret = foci_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def foci_resample(
    foci_in: InputPathType,
    foci_out: str,
    left_surfaces: FociResampleLeftSurfacesParamsDict | None = None,
    right_surfaces: FociResampleRightSurfacesParamsDict | None = None,
    cerebellum_surfaces: FociResampleCerebellumSurfacesParamsDict | None = None,
    restore_xyz: bool = False,
    discard_distance_from_surface: bool = False,
    runner: Runner | None = None,
) -> FociResampleOutputs:
    """
    PROJECT FOCI TO A DIFFERENT SURFACE.
    
    Unprojects foci from the <current-surf> for the structure, then projects
    them to <new-surf>. If the foci have meaningful distances above or below the
    surface, use anatomical surfaces. If the foci should be on the surface, use
    registered spheres and the options -discard-distance-from-surface and
    -restore-xyz.
    
    Args:
        foci_in: the input foci file.
        foci_out: the output foci file.
        left_surfaces: the left surfaces for resampling.
        right_surfaces: the right surfaces for resampling.
        cerebellum_surfaces: the cerebellum surfaces for resampling.
        restore_xyz: put the original xyz coordinates into the foci, rather\
            than the coordinates obtained from unprojection.
        discard_distance_from_surface: ignore the distance the foci are above\
            or below the current surface.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FociResampleOutputs`).
    """
    params = foci_resample_params(
        foci_in=foci_in,
        foci_out=foci_out,
        left_surfaces=left_surfaces,
        right_surfaces=right_surfaces,
        cerebellum_surfaces=cerebellum_surfaces,
        restore_xyz=restore_xyz,
        discard_distance_from_surface=discard_distance_from_surface,
    )
    return foci_resample_execute(params, runner)


__all__ = [
    "FOCI_RESAMPLE_METADATA",
    "FociResampleCerebellumSurfacesParamsDict",
    "FociResampleCerebellumSurfacesParamsDictTagged",
    "FociResampleLeftSurfacesParamsDict",
    "FociResampleLeftSurfacesParamsDictTagged",
    "FociResampleOutputs",
    "FociResampleParamsDict",
    "FociResampleParamsDictTagged",
    "FociResampleRightSurfacesParamsDict",
    "FociResampleRightSurfacesParamsDictTagged",
    "foci_resample",
    "foci_resample_cerebellum_surfaces",
    "foci_resample_execute",
    "foci_resample_left_surfaces",
    "foci_resample_params",
    "foci_resample_right_surfaces",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_MASK_METADATA = Metadata(
    id="681850b8635d1fb30c276619483f247b1d36e566.workbench",
    name="metric-mask",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricMaskParamsDictNoTag = typing.TypedDict('_MetricMaskParamsDictNoTag', {
    "metric": InputPathType,
    "mask": InputPathType,
    "metric-out": str,
    "column": typing.NotRequired[str | None],
})
MetricMaskParamsDictTagged = typing.TypedDict('MetricMaskParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-mask"],
    "metric": InputPathType,
    "mask": InputPathType,
    "metric-out": str,
    "column": typing.NotRequired[str | None],
})
MetricMaskParamsDict = _MetricMaskParamsDictNoTag | MetricMaskParamsDictTagged


class MetricMaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricMaskParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_mask_params(
    metric: InputPathType,
    mask: InputPathType,
    metric_out: str,
    column: str | None = None,
) -> MetricMaskParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the input metric.
        mask: the mask metric.
        metric_out: the output metric.
        column: select a single column\
            \
            the column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-mask",
        "metric": metric,
        "mask": mask,
        "metric-out": metric_out,
    }
    if column is not None:
        params["column"] = column
    return params


def metric_mask_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMaskParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("mask", None) is None:
        raise StyxValidationError("`mask` must not be None")
    if not isinstance(params["mask"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')


def metric_mask_cargs(
    params: MetricMaskParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-mask"
    ])
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("mask", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    return cargs


def metric_mask_outputs(
    params: MetricMaskParamsDict,
    execution: Execution,
) -> MetricMaskOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricMaskOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_mask_execute(
    params: MetricMaskParamsDict,
    runner: Runner | None = None,
) -> MetricMaskOutputs:
    """
    MASK A METRIC FILE.
    
    By default, the output metric is a copy of the input metric, but with zeros
    wherever the mask metric is zero or negative. if -column is specified, the
    output contains only one column, the masked version of the specified input
    column.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMaskOutputs`).
    """
    metric_mask_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_MASK_METADATA)
    params = execution.params(params)
    cargs = metric_mask_cargs(params, execution)
    ret = metric_mask_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_mask(
    metric: InputPathType,
    mask: InputPathType,
    metric_out: str,
    column: str | None = None,
    runner: Runner | None = None,
) -> MetricMaskOutputs:
    """
    MASK A METRIC FILE.
    
    By default, the output metric is a copy of the input metric, but with zeros
    wherever the mask metric is zero or negative. if -column is specified, the
    output contains only one column, the masked version of the specified input
    column.
    
    Args:
        metric: the input metric.
        mask: the mask metric.
        metric_out: the output metric.
        column: select a single column\
            \
            the column number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMaskOutputs`).
    """
    params = metric_mask_params(
        metric=metric,
        mask=mask,
        metric_out=metric_out,
        column=column,
    )
    return metric_mask_execute(params, runner)


__all__ = [
    "METRIC_MASK_METADATA",
    "MetricMaskOutputs",
    "MetricMaskParamsDict",
    "MetricMaskParamsDictTagged",
    "metric_mask",
    "metric_mask_execute",
    "metric_mask_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_COPY_EXTENSIONS_METADATA = Metadata(
    id="ce53186c774fd3df17402a86376908e43ca4e439.workbench",
    name="volume-copy-extensions",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeCopyExtensionsParamsDictNoTag = typing.TypedDict('_VolumeCopyExtensionsParamsDictNoTag', {
    "data-volume": InputPathType,
    "extension-volume": InputPathType,
    "volume-out": str,
    "drop-unknown": bool,
})
VolumeCopyExtensionsParamsDictTagged = typing.TypedDict('VolumeCopyExtensionsParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-copy-extensions"],
    "data-volume": InputPathType,
    "extension-volume": InputPathType,
    "volume-out": str,
    "drop-unknown": bool,
})
VolumeCopyExtensionsParamsDict = _VolumeCopyExtensionsParamsDictNoTag | VolumeCopyExtensionsParamsDictTagged


class VolumeCopyExtensionsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeCopyExtensionsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_copy_extensions_params(
    data_volume: InputPathType,
    extension_volume: InputPathType,
    volume_out: str,
    drop_unknown: bool = False,
) -> VolumeCopyExtensionsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_volume: the volume file containing the voxel data to use.
        extension_volume: the volume file containing the extensions to use.
        volume_out: the output volume.
        drop_unknown: don't copy extensions that workbench doesn't understand.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-copy-extensions",
        "data-volume": data_volume,
        "extension-volume": extension_volume,
        "volume-out": volume_out,
        "drop-unknown": drop_unknown,
    }
    return params


def volume_copy_extensions_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeCopyExtensionsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-volume", None) is None:
        raise StyxValidationError("`data-volume` must not be None")
    if not isinstance(params["data-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`data-volume` has the wrong type: Received `{type(params.get("data-volume", None))}` expected `InputPathType`')
    if params.get("extension-volume", None) is None:
        raise StyxValidationError("`extension-volume` must not be None")
    if not isinstance(params["extension-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`extension-volume` has the wrong type: Received `{type(params.get("extension-volume", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("drop-unknown", False) is None:
        raise StyxValidationError("`drop-unknown` must not be None")
    if not isinstance(params["drop-unknown"], bool):
        raise StyxValidationError(f'`drop-unknown` has the wrong type: Received `{type(params.get("drop-unknown", False))}` expected `bool`')


def volume_copy_extensions_cargs(
    params: VolumeCopyExtensionsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-copy-extensions"
    ])
    cargs.append(execution.input_file(params.get("data-volume", None)))
    cargs.append(execution.input_file(params.get("extension-volume", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("drop-unknown", False):
        cargs.append("-drop-unknown")
    return cargs


def volume_copy_extensions_outputs(
    params: VolumeCopyExtensionsParamsDict,
    execution: Execution,
) -> VolumeCopyExtensionsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeCopyExtensionsOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_copy_extensions_execute(
    params: VolumeCopyExtensionsParamsDict,
    runner: Runner | None = None,
) -> VolumeCopyExtensionsOutputs:
    """
    COPY EXTENDED DATA TO ANOTHER VOLUME FILE.
    
    This command copies the information in a volume file that isn't a critical
    part of the standard header or data matrix, e.g. map names, palette
    settings, label tables. If -drop-unknown is not specified, it also copies
    similar kinds of information set by other software.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCopyExtensionsOutputs`).
    """
    volume_copy_extensions_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_COPY_EXTENSIONS_METADATA)
    params = execution.params(params)
    cargs = volume_copy_extensions_cargs(params, execution)
    ret = volume_copy_extensions_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_copy_extensions(
    data_volume: InputPathType,
    extension_volume: InputPathType,
    volume_out: str,
    drop_unknown: bool = False,
    runner: Runner | None = None,
) -> VolumeCopyExtensionsOutputs:
    """
    COPY EXTENDED DATA TO ANOTHER VOLUME FILE.
    
    This command copies the information in a volume file that isn't a critical
    part of the standard header or data matrix, e.g. map names, palette
    settings, label tables. If -drop-unknown is not specified, it also copies
    similar kinds of information set by other software.
    
    Args:
        data_volume: the volume file containing the voxel data to use.
        extension_volume: the volume file containing the extensions to use.
        volume_out: the output volume.
        drop_unknown: don't copy extensions that workbench doesn't understand.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCopyExtensionsOutputs`).
    """
    params = volume_copy_extensions_params(
        data_volume=data_volume,
        extension_volume=extension_volume,
        volume_out=volume_out,
        drop_unknown=drop_unknown,
    )
    return volume_copy_extensions_execute(params, runner)


__all__ = [
    "VOLUME_COPY_EXTENSIONS_METADATA",
    "VolumeCopyExtensionsOutputs",
    "VolumeCopyExtensionsParamsDict",
    "VolumeCopyExtensionsParamsDictTagged",
    "volume_copy_extensions",
    "volume_copy_extensions_execute",
    "volume_copy_extensions_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_EXTREMA_METADATA = Metadata(
    id="96cc4edc754b1196eb76c4a8761c2b9a9cc4593f.workbench",
    name="cifti-extrema",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiExtremaThresholdParamsDictNoTag = typing.TypedDict('_CiftiExtremaThresholdParamsDictNoTag', {
    "low": float,
    "high": float,
})
CiftiExtremaThresholdParamsDictTagged = typing.TypedDict('CiftiExtremaThresholdParamsDictTagged', {
    "@type": typing.Literal["threshold"],
    "low": float,
    "high": float,
})
CiftiExtremaThresholdParamsDict = _CiftiExtremaThresholdParamsDictNoTag | CiftiExtremaThresholdParamsDictTagged


_CiftiExtremaParamsDictNoTag = typing.TypedDict('_CiftiExtremaParamsDictNoTag', {
    "cifti": InputPathType,
    "surface-distance": float,
    "volume-distance": float,
    "direction": str,
    "cifti-out": str,
    "threshold": typing.NotRequired[CiftiExtremaThresholdParamsDict | None],
    "volume-kernel": typing.NotRequired[float | None],
    "surface-kernel": typing.NotRequired[float | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "only-minima": bool,
    "only-maxima": bool,
    "consolidate-mode": bool,
    "sum-maps": bool,
    "merged-volume": bool,
    "presmooth-fwhm": bool,
})
CiftiExtremaParamsDictTagged = typing.TypedDict('CiftiExtremaParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-extrema"],
    "cifti": InputPathType,
    "surface-distance": float,
    "volume-distance": float,
    "direction": str,
    "cifti-out": str,
    "threshold": typing.NotRequired[CiftiExtremaThresholdParamsDict | None],
    "volume-kernel": typing.NotRequired[float | None],
    "surface-kernel": typing.NotRequired[float | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "only-minima": bool,
    "only-maxima": bool,
    "consolidate-mode": bool,
    "sum-maps": bool,
    "merged-volume": bool,
    "presmooth-fwhm": bool,
})
CiftiExtremaParamsDict = _CiftiExtremaParamsDictNoTag | CiftiExtremaParamsDictTagged


def cifti_extrema_threshold(
    low: float,
    high: float,
) -> CiftiExtremaThresholdParamsDictTagged:
    """
    Build parameters.
    
    Args:
        low: the largest value to consider for being a minimum.
        high: the smallest value to consider for being a maximum.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "threshold",
        "low": low,
        "high": high,
    }
    return params


def cifti_extrema_threshold_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExtremaThresholdParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("low", None) is None:
        raise StyxValidationError("`low` must not be None")
    if not isinstance(params["low"], (float, int)):
        raise StyxValidationError(f'`low` has the wrong type: Received `{type(params.get("low", None))}` expected `float`')
    if params.get("high", None) is None:
        raise StyxValidationError("`high` must not be None")
    if not isinstance(params["high"], (float, int)):
        raise StyxValidationError(f'`high` has the wrong type: Received `{type(params.get("high", None))}` expected `float`')


def cifti_extrema_threshold_cargs(
    params: CiftiExtremaThresholdParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-threshold",
        str(params.get("low", None)),
        str(params.get("high", None))
    ])
    return cargs


class CiftiExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiExtremaParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_extrema_params(
    cifti: InputPathType,
    surface_distance: float,
    volume_distance: float,
    direction: str,
    cifti_out: str,
    threshold: CiftiExtremaThresholdParamsDict | None = None,
    volume_kernel: float | None = None,
    surface_kernel: float | None = None,
    surface: InputPathType | None = None,
    surface_: InputPathType | None = None,
    surface_2: InputPathType | None = None,
    only_minima: bool = False,
    only_maxima: bool = False,
    consolidate_mode: bool = False,
    sum_maps: bool = False,
    merged_volume: bool = False,
    presmooth_fwhm: bool = False,
) -> CiftiExtremaParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti.
        surface_distance: the minimum distance between extrema of the same\
            type, for surface components.
        volume_distance: the minimum distance between extrema of the same type,\
            for volume components.
        direction: which dimension to find extrema along, ROW or COLUMN.
        cifti_out: the output cifti.
        threshold: ignore small extrema.
        volume_kernel: smooth volume components before finding extrema\
            \
            the size of the gaussian volume smoothing kernel in mm, as sigma by\
            default.
        surface_kernel: smooth on the surface before finding extrema\
            \
            the size of the gaussian surface smoothing kernel in mm, as sigma\
            by default.
        surface: specify the cerebellum surface to use\
            \
            the cerebellum surface file.
        surface_: specify the right surface to use\
            \
            the right surface file.
        surface_2: specify the left surface to use\
            \
            the left surface file.
        only_minima: only find the minima.
        only_maxima: only find the maxima.
        consolidate_mode: use consolidation of local minima instead of a large\
            neighborhood.
        sum_maps: output the sum of the extrema maps instead of each map\
            separately.
        merged_volume: treat volume components as if they were a single\
            component.
        presmooth_fwhm: smoothing kernel distances are FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-extrema",
        "cifti": cifti,
        "surface-distance": surface_distance,
        "volume-distance": volume_distance,
        "direction": direction,
        "cifti-out": cifti_out,
        "only-minima": only_minima,
        "only-maxima": only_maxima,
        "consolidate-mode": consolidate_mode,
        "sum-maps": sum_maps,
        "merged-volume": merged_volume,
        "presmooth-fwhm": presmooth_fwhm,
    }
    if threshold is not None:
        params["threshold"] = threshold
    if volume_kernel is not None:
        params["volume-kernel"] = volume_kernel
    if surface_kernel is not None:
        params["surface-kernel"] = surface_kernel
    if surface is not None:
        params["surface"] = surface
    if surface_ is not None:
        params["surface"] = surface_
    if surface_2 is not None:
        params["surface"] = surface_2
    return params


def cifti_extrema_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiExtremaParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("surface-distance", None) is None:
        raise StyxValidationError("`surface-distance` must not be None")
    if not isinstance(params["surface-distance"], (float, int)):
        raise StyxValidationError(f'`surface-distance` has the wrong type: Received `{type(params.get("surface-distance", None))}` expected `float`')
    if params.get("volume-distance", None) is None:
        raise StyxValidationError("`volume-distance` must not be None")
    if not isinstance(params["volume-distance"], (float, int)):
        raise StyxValidationError(f'`volume-distance` has the wrong type: Received `{type(params.get("volume-distance", None))}` expected `float`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("threshold", None) is not None:
        cifti_extrema_threshold_validate(params["threshold"])
    if params.get("volume-kernel", None) is not None:
        if not isinstance(params["volume-kernel"], (float, int)):
            raise StyxValidationError(f'`volume-kernel` has the wrong type: Received `{type(params.get("volume-kernel", None))}` expected `float | None`')
    if params.get("surface-kernel", None) is not None:
        if not isinstance(params["surface-kernel"], (float, int)):
            raise StyxValidationError(f'`surface-kernel` has the wrong type: Received `{type(params.get("surface-kernel", None))}` expected `float | None`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')
    if params.get("only-minima", False) is None:
        raise StyxValidationError("`only-minima` must not be None")
    if not isinstance(params["only-minima"], bool):
        raise StyxValidationError(f'`only-minima` has the wrong type: Received `{type(params.get("only-minima", False))}` expected `bool`')
    if params.get("only-maxima", False) is None:
        raise StyxValidationError("`only-maxima` must not be None")
    if not isinstance(params["only-maxima"], bool):
        raise StyxValidationError(f'`only-maxima` has the wrong type: Received `{type(params.get("only-maxima", False))}` expected `bool`')
    if params.get("consolidate-mode", False) is None:
        raise StyxValidationError("`consolidate-mode` must not be None")
    if not isinstance(params["consolidate-mode"], bool):
        raise StyxValidationError(f'`consolidate-mode` has the wrong type: Received `{type(params.get("consolidate-mode", False))}` expected `bool`')
    if params.get("sum-maps", False) is None:
        raise StyxValidationError("`sum-maps` must not be None")
    if not isinstance(params["sum-maps"], bool):
        raise StyxValidationError(f'`sum-maps` has the wrong type: Received `{type(params.get("sum-maps", False))}` expected `bool`')
    if params.get("merged-volume", False) is None:
        raise StyxValidationError("`merged-volume` must not be None")
    if not isinstance(params["merged-volume"], bool):
        raise StyxValidationError(f'`merged-volume` has the wrong type: Received `{type(params.get("merged-volume", False))}` expected `bool`')
    if params.get("presmooth-fwhm", False) is None:
        raise StyxValidationError("`presmooth-fwhm` must not be None")
    if not isinstance(params["presmooth-fwhm"], bool):
        raise StyxValidationError(f'`presmooth-fwhm` has the wrong type: Received `{type(params.get("presmooth-fwhm", False))}` expected `bool`')


def cifti_extrema_cargs(
    params: CiftiExtremaParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-extrema"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(str(params.get("surface-distance", None)))
    cargs.append(str(params.get("volume-distance", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("threshold", None) is not None:
        cargs.extend(cifti_extrema_threshold_cargs(params.get("threshold", None), execution))
    if params.get("volume-kernel", None) is not None:
        cargs.extend([
            "-volume-presmooth",
            str(params.get("volume-kernel", None))
        ])
    if params.get("surface-kernel", None) is not None:
        cargs.extend([
            "-surface-presmooth",
            str(params.get("surface-kernel", None))
        ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-cerebellum-surface",
            execution.input_file(params.get("surface", None))
        ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-right-surface",
            execution.input_file(params.get("surface", None))
        ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-left-surface",
            execution.input_file(params.get("surface", None))
        ])
    if params.get("only-minima", False):
        cargs.append("-only-minima")
    if params.get("only-maxima", False):
        cargs.append("-only-maxima")
    if params.get("consolidate-mode", False):
        cargs.append("-consolidate-mode")
    if params.get("sum-maps", False):
        cargs.append("-sum-maps")
    if params.get("merged-volume", False):
        cargs.append("-merged-volume")
    if params.get("presmooth-fwhm", False):
        cargs.append("-presmooth-fwhm")
    return cargs


def cifti_extrema_outputs(
    params: CiftiExtremaParamsDict,
    execution: Execution,
) -> CiftiExtremaOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiExtremaOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_extrema_execute(
    params: CiftiExtremaParamsDict,
    runner: Runner | None = None,
) -> CiftiExtremaOutputs:
    """
    FIND EXTREMA IN A CIFTI FILE.
    
    Finds spatial locations in a cifti file that have more extreme values than
    all nearby locations in the same component (surface or volume structure).
    The input cifti file must have a brain models mapping along the specified
    direction. COLUMN is the direction that works on dtseries and dscalar. For
    dconn, if it is symmetric use COLUMN, otherwise use ROW.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiExtremaOutputs`).
    """
    cifti_extrema_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_EXTREMA_METADATA)
    params = execution.params(params)
    cargs = cifti_extrema_cargs(params, execution)
    ret = cifti_extrema_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_extrema(
    cifti: InputPathType,
    surface_distance: float,
    volume_distance: float,
    direction: str,
    cifti_out: str,
    threshold: CiftiExtremaThresholdParamsDict | None = None,
    volume_kernel: float | None = None,
    surface_kernel: float | None = None,
    surface: InputPathType | None = None,
    surface_: InputPathType | None = None,
    surface_2: InputPathType | None = None,
    only_minima: bool = False,
    only_maxima: bool = False,
    consolidate_mode: bool = False,
    sum_maps: bool = False,
    merged_volume: bool = False,
    presmooth_fwhm: bool = False,
    runner: Runner | None = None,
) -> CiftiExtremaOutputs:
    """
    FIND EXTREMA IN A CIFTI FILE.
    
    Finds spatial locations in a cifti file that have more extreme values than
    all nearby locations in the same component (surface or volume structure).
    The input cifti file must have a brain models mapping along the specified
    direction. COLUMN is the direction that works on dtseries and dscalar. For
    dconn, if it is symmetric use COLUMN, otherwise use ROW.
    
    Args:
        cifti: the input cifti.
        surface_distance: the minimum distance between extrema of the same\
            type, for surface components.
        volume_distance: the minimum distance between extrema of the same type,\
            for volume components.
        direction: which dimension to find extrema along, ROW or COLUMN.
        cifti_out: the output cifti.
        threshold: ignore small extrema.
        volume_kernel: smooth volume components before finding extrema\
            \
            the size of the gaussian volume smoothing kernel in mm, as sigma by\
            default.
        surface_kernel: smooth on the surface before finding extrema\
            \
            the size of the gaussian surface smoothing kernel in mm, as sigma\
            by default.
        surface: specify the cerebellum surface to use\
            \
            the cerebellum surface file.
        surface_: specify the right surface to use\
            \
            the right surface file.
        surface_2: specify the left surface to use\
            \
            the left surface file.
        only_minima: only find the minima.
        only_maxima: only find the maxima.
        consolidate_mode: use consolidation of local minima instead of a large\
            neighborhood.
        sum_maps: output the sum of the extrema maps instead of each map\
            separately.
        merged_volume: treat volume components as if they were a single\
            component.
        presmooth_fwhm: smoothing kernel distances are FWHM, not sigma.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiExtremaOutputs`).
    """
    params = cifti_extrema_params(
        cifti=cifti,
        surface_distance=surface_distance,
        volume_distance=volume_distance,
        direction=direction,
        cifti_out=cifti_out,
        threshold=threshold,
        volume_kernel=volume_kernel,
        surface_kernel=surface_kernel,
        surface=surface,
        surface_=surface_,
        surface_2=surface_2,
        only_minima=only_minima,
        only_maxima=only_maxima,
        consolidate_mode=consolidate_mode,
        sum_maps=sum_maps,
        merged_volume=merged_volume,
        presmooth_fwhm=presmooth_fwhm,
    )
    return cifti_extrema_execute(params, runner)


__all__ = [
    "CIFTI_EXTREMA_METADATA",
    "CiftiExtremaOutputs",
    "CiftiExtremaParamsDict",
    "CiftiExtremaParamsDictTagged",
    "CiftiExtremaThresholdParamsDict",
    "CiftiExtremaThresholdParamsDictTagged",
    "cifti_extrema",
    "cifti_extrema_execute",
    "cifti_extrema_params",
    "cifti_extrema_threshold",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FILE_CONVERT_METADATA = Metadata(
    id="955bdf431ca1cccbf73a19afbeac8667fbb4a28c.workbench",
    name="file-convert",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_FileConvertBorderVersionConvertParamsDictNoTag = typing.TypedDict('_FileConvertBorderVersionConvertParamsDictNoTag', {
    "border-in": InputPathType,
    "out-version": int,
    "border-out": str,
    "surface": typing.NotRequired[InputPathType | None],
})
FileConvertBorderVersionConvertParamsDictTagged = typing.TypedDict('FileConvertBorderVersionConvertParamsDictTagged', {
    "@type": typing.Literal["border-version-convert"],
    "border-in": InputPathType,
    "out-version": int,
    "border-out": str,
    "surface": typing.NotRequired[InputPathType | None],
})
FileConvertBorderVersionConvertParamsDict = _FileConvertBorderVersionConvertParamsDictNoTag | FileConvertBorderVersionConvertParamsDictTagged


_FileConvertNiftiVersionConvertParamsDictNoTag = typing.TypedDict('_FileConvertNiftiVersionConvertParamsDictNoTag', {
    "input": str,
    "version": int,
    "output": str,
})
FileConvertNiftiVersionConvertParamsDictTagged = typing.TypedDict('FileConvertNiftiVersionConvertParamsDictTagged', {
    "@type": typing.Literal["nifti-version-convert"],
    "input": str,
    "version": int,
    "output": str,
})
FileConvertNiftiVersionConvertParamsDict = _FileConvertNiftiVersionConvertParamsDictNoTag | FileConvertNiftiVersionConvertParamsDictTagged


_FileConvertCiftiVersionConvertParamsDictNoTag = typing.TypedDict('_FileConvertCiftiVersionConvertParamsDictNoTag', {
    "cifti-in": InputPathType,
    "version": str,
    "cifti-out": str,
})
FileConvertCiftiVersionConvertParamsDictTagged = typing.TypedDict('FileConvertCiftiVersionConvertParamsDictTagged', {
    "@type": typing.Literal["cifti-version-convert"],
    "cifti-in": InputPathType,
    "version": str,
    "cifti-out": str,
})
FileConvertCiftiVersionConvertParamsDict = _FileConvertCiftiVersionConvertParamsDictNoTag | FileConvertCiftiVersionConvertParamsDictTagged


_FileConvertParamsDictNoTag = typing.TypedDict('_FileConvertParamsDictNoTag', {
    "border-version-convert": typing.NotRequired[FileConvertBorderVersionConvertParamsDict | None],
    "nifti-version-convert": typing.NotRequired[FileConvertNiftiVersionConvertParamsDict | None],
    "cifti-version-convert": typing.NotRequired[FileConvertCiftiVersionConvertParamsDict | None],
})
FileConvertParamsDictTagged = typing.TypedDict('FileConvertParamsDictTagged', {
    "@type": typing.Literal["workbench/file-convert"],
    "border-version-convert": typing.NotRequired[FileConvertBorderVersionConvertParamsDict | None],
    "nifti-version-convert": typing.NotRequired[FileConvertNiftiVersionConvertParamsDict | None],
    "cifti-version-convert": typing.NotRequired[FileConvertCiftiVersionConvertParamsDict | None],
})
FileConvertParamsDict = _FileConvertParamsDictNoTag | FileConvertParamsDictTagged


def file_convert_border_version_convert(
    border_in: InputPathType,
    out_version: int,
    border_out: str,
    surface: InputPathType | None = None,
) -> FileConvertBorderVersionConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border_in: the input border file.
        out_version: the format version to write as, 1 or 3 (2 doesn't exist).
        border_out: output - the output border file.
        surface: must be specified if the input is version 1\
            \
            use this surface file for structure and number of vertices, ignore\
            borders on other structures.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "border-version-convert",
        "border-in": border_in,
        "out-version": out_version,
        "border-out": border_out,
    }
    if surface is not None:
        params["surface"] = surface
    return params


def file_convert_border_version_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileConvertBorderVersionConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border-in", None) is None:
        raise StyxValidationError("`border-in` must not be None")
    if not isinstance(params["border-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border-in` has the wrong type: Received `{type(params.get("border-in", None))}` expected `InputPathType`')
    if params.get("out-version", None) is None:
        raise StyxValidationError("`out-version` must not be None")
    if not isinstance(params["out-version"], int):
        raise StyxValidationError(f'`out-version` has the wrong type: Received `{type(params.get("out-version", None))}` expected `int`')
    if params.get("border-out", None) is None:
        raise StyxValidationError("`border-out` must not be None")
    if not isinstance(params["border-out"], str):
        raise StyxValidationError(f'`border-out` has the wrong type: Received `{type(params.get("border-out", None))}` expected `str`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')


def file_convert_border_version_convert_cargs(
    params: FileConvertBorderVersionConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-border-version-convert",
        execution.input_file(params.get("border-in", None)),
        str(params.get("out-version", None)),
        params.get("border-out", None)
    ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-surface",
            execution.input_file(params.get("surface", None))
        ])
    return cargs


def file_convert_nifti_version_convert(
    input_: str,
    version: int,
    output: str,
) -> FileConvertNiftiVersionConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input nifti file.
        version: the nifti version to write as.
        output: output - the output nifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "nifti-version-convert",
        "input": input_,
        "version": version,
        "output": output,
    }
    return params


def file_convert_nifti_version_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileConvertNiftiVersionConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], str):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `str`')
    if params.get("version", None) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], int):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", None))}` expected `int`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def file_convert_nifti_version_convert_cargs(
    params: FileConvertNiftiVersionConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-nifti-version-convert",
        params.get("input", None),
        str(params.get("version", None)),
        params.get("output", None)
    ])
    return cargs


def file_convert_cifti_version_convert(
    cifti_in: InputPathType,
    version: str,
    cifti_out: str,
) -> FileConvertCiftiVersionConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        version: the cifti version to write as.
        cifti_out: output - the output cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti-version-convert",
        "cifti-in": cifti_in,
        "version": version,
        "cifti-out": cifti_out,
    }
    return params


def file_convert_cifti_version_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileConvertCiftiVersionConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("version", None) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], str):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')


def file_convert_cifti_version_convert_cargs(
    params: FileConvertCiftiVersionConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti-version-convert",
        execution.input_file(params.get("cifti-in", None)),
        params.get("version", None),
        params.get("cifti-out", None)
    ])
    return cargs


class FileConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FileConvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def file_convert_params(
    border_version_convert: FileConvertBorderVersionConvertParamsDict | None = None,
    nifti_version_convert: FileConvertNiftiVersionConvertParamsDict | None = None,
    cifti_version_convert: FileConvertCiftiVersionConvertParamsDict | None = None,
) -> FileConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border_version_convert: write a border file with a different version.
        nifti_version_convert: write a nifti file with a different version.
        cifti_version_convert: write a cifti file with a different version.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/file-convert",
    }
    if border_version_convert is not None:
        params["border-version-convert"] = border_version_convert
    if nifti_version_convert is not None:
        params["nifti-version-convert"] = nifti_version_convert
    if cifti_version_convert is not None:
        params["cifti-version-convert"] = cifti_version_convert
    return params


def file_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FileConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border-version-convert", None) is not None:
        file_convert_border_version_convert_validate(params["border-version-convert"])
    if params.get("nifti-version-convert", None) is not None:
        file_convert_nifti_version_convert_validate(params["nifti-version-convert"])
    if params.get("cifti-version-convert", None) is not None:
        file_convert_cifti_version_convert_validate(params["cifti-version-convert"])


def file_convert_cargs(
    params: FileConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-file-convert"
    ])
    if params.get("border-version-convert", None) is not None or params.get("nifti-version-convert", None) is not None or params.get("cifti-version-convert", None) is not None:
        cargs.extend([
            *(file_convert_border_version_convert_cargs(params.get("border-version-convert", None), execution) if (params.get("border-version-convert", None) is not None) else []),
            *(file_convert_nifti_version_convert_cargs(params.get("nifti-version-convert", None), execution) if (params.get("nifti-version-convert", None) is not None) else []),
            *(file_convert_cifti_version_convert_cargs(params.get("cifti-version-convert", None), execution) if (params.get("cifti-version-convert", None) is not None) else [])
        ])
    return cargs


def file_convert_outputs(
    params: FileConvertParamsDict,
    execution: Execution,
) -> FileConvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FileConvertOutputs(
        root=execution.output_file("."),
    )
    return ret


def file_convert_execute(
    params: FileConvertParamsDict,
    runner: Runner | None = None,
) -> FileConvertOutputs:
    """
    CHANGE VERSION OF FILE FORMAT.
    
    You may only specify one top-level option.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileConvertOutputs`).
    """
    file_convert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FILE_CONVERT_METADATA)
    params = execution.params(params)
    cargs = file_convert_cargs(params, execution)
    ret = file_convert_outputs(params, execution)
    execution.run(cargs)
    return ret


def file_convert(
    border_version_convert: FileConvertBorderVersionConvertParamsDict | None = None,
    nifti_version_convert: FileConvertNiftiVersionConvertParamsDict | None = None,
    cifti_version_convert: FileConvertCiftiVersionConvertParamsDict | None = None,
    runner: Runner | None = None,
) -> FileConvertOutputs:
    """
    CHANGE VERSION OF FILE FORMAT.
    
    You may only specify one top-level option.
    
    Args:
        border_version_convert: write a border file with a different version.
        nifti_version_convert: write a nifti file with a different version.
        cifti_version_convert: write a cifti file with a different version.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileConvertOutputs`).
    """
    params = file_convert_params(
        border_version_convert=border_version_convert,
        nifti_version_convert=nifti_version_convert,
        cifti_version_convert=cifti_version_convert,
    )
    return file_convert_execute(params, runner)


__all__ = [
    "FILE_CONVERT_METADATA",
    "FileConvertBorderVersionConvertParamsDict",
    "FileConvertBorderVersionConvertParamsDictTagged",
    "FileConvertCiftiVersionConvertParamsDict",
    "FileConvertCiftiVersionConvertParamsDictTagged",
    "FileConvertNiftiVersionConvertParamsDict",
    "FileConvertNiftiVersionConvertParamsDictTagged",
    "FileConvertOutputs",
    "FileConvertParamsDict",
    "FileConvertParamsDictTagged",
    "file_convert",
    "file_convert_border_version_convert",
    "file_convert_cifti_version_convert",
    "file_convert_execute",
    "file_convert_nifti_version_convert",
    "file_convert_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_MATCH_METADATA = Metadata(
    id="6335c2c3b01da0e8439ffac4c9d4032ef21fd455.workbench",
    name="surface-match",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceMatchParamsDictNoTag = typing.TypedDict('_SurfaceMatchParamsDictNoTag', {
    "Match Surface File": InputPathType,
    "Input Surface File": InputPathType,
    "Output Surface Name": str,
})
SurfaceMatchParamsDictTagged = typing.TypedDict('SurfaceMatchParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-match"],
    "Match Surface File": InputPathType,
    "Input Surface File": InputPathType,
    "Output Surface Name": str,
})
SurfaceMatchParamsDict = _SurfaceMatchParamsDictNoTag | SurfaceMatchParamsDictTagged


class SurfaceMatchOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceMatchParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def surface_match_params(
    match_surface_file: InputPathType,
    input_surface_file: InputPathType,
    output_surface_name: str,
) -> SurfaceMatchParamsDictTagged:
    """
    Build parameters.
    
    Args:
        match_surface_file: Match (Reference) Surface.
        input_surface_file: File containing surface that will be transformed.
        output_surface_name: Surface File after transformation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-match",
        "Match Surface File": match_surface_file,
        "Input Surface File": input_surface_file,
        "Output Surface Name": output_surface_name,
    }
    return params


def surface_match_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceMatchParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("Match Surface File", None) is None:
        raise StyxValidationError("`Match Surface File` must not be None")
    if not isinstance(params["Match Surface File"], (pathlib.Path, str)):
        raise StyxValidationError(f'`Match Surface File` has the wrong type: Received `{type(params.get("Match Surface File", None))}` expected `InputPathType`')
    if params.get("Input Surface File", None) is None:
        raise StyxValidationError("`Input Surface File` must not be None")
    if not isinstance(params["Input Surface File"], (pathlib.Path, str)):
        raise StyxValidationError(f'`Input Surface File` has the wrong type: Received `{type(params.get("Input Surface File", None))}` expected `InputPathType`')
    if params.get("Output Surface Name", None) is None:
        raise StyxValidationError("`Output Surface Name` must not be None")
    if not isinstance(params["Output Surface Name"], str):
        raise StyxValidationError(f'`Output Surface Name` has the wrong type: Received `{type(params.get("Output Surface Name", None))}` expected `str`')


def surface_match_cargs(
    params: SurfaceMatchParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-match"
    ])
    cargs.append(execution.input_file(params.get("Match Surface File", None)))
    cargs.append(execution.input_file(params.get("Input Surface File", None)))
    cargs.append(params.get("Output Surface Name", None))
    return cargs


def surface_match_outputs(
    params: SurfaceMatchParamsDict,
    execution: Execution,
) -> SurfaceMatchOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceMatchOutputs(
        root=execution.output_file("."),
    )
    return ret


def surface_match_execute(
    params: SurfaceMatchParamsDict,
    runner: Runner | None = None,
) -> SurfaceMatchOutputs:
    """
    SURFACE MATCH.
    
    The Input Surface File will be transformed so that its coordinate ranges
    (bounding box) match that of the Match Surface File.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceMatchOutputs`).
    """
    surface_match_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_MATCH_METADATA)
    params = execution.params(params)
    cargs = surface_match_cargs(params, execution)
    ret = surface_match_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_match(
    match_surface_file: InputPathType,
    input_surface_file: InputPathType,
    output_surface_name: str,
    runner: Runner | None = None,
) -> SurfaceMatchOutputs:
    """
    SURFACE MATCH.
    
    The Input Surface File will be transformed so that its coordinate ranges
    (bounding box) match that of the Match Surface File.
    
    Args:
        match_surface_file: Match (Reference) Surface.
        input_surface_file: File containing surface that will be transformed.
        output_surface_name: Surface File after transformation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceMatchOutputs`).
    """
    params = surface_match_params(
        match_surface_file=match_surface_file,
        input_surface_file=input_surface_file,
        output_surface_name=output_surface_name,
    )
    return surface_match_execute(params, runner)


__all__ = [
    "SURFACE_MATCH_METADATA",
    "SurfaceMatchOutputs",
    "SurfaceMatchParamsDict",
    "SurfaceMatchParamsDictTagged",
    "surface_match",
    "surface_match_execute",
    "surface_match_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_PAIRWISE_CORRELATION_METADATA = Metadata(
    id="ad1c3d662547315bfc0576b17cb38a01d7cddf48.workbench",
    name="cifti-pairwise-correlation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiPairwiseCorrelationParamsDictNoTag = typing.TypedDict('_CiftiPairwiseCorrelationParamsDictNoTag', {
    "cifti-a": InputPathType,
    "cifti-b": InputPathType,
    "cifti-out": str,
    "override-mapping-check": bool,
    "fisher-z": bool,
})
CiftiPairwiseCorrelationParamsDictTagged = typing.TypedDict('CiftiPairwiseCorrelationParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-pairwise-correlation"],
    "cifti-a": InputPathType,
    "cifti-b": InputPathType,
    "cifti-out": str,
    "override-mapping-check": bool,
    "fisher-z": bool,
})
CiftiPairwiseCorrelationParamsDict = _CiftiPairwiseCorrelationParamsDictNoTag | CiftiPairwiseCorrelationParamsDictTagged


class CiftiPairwiseCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiPairwiseCorrelationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_pairwise_correlation_params(
    cifti_a: InputPathType,
    cifti_b: InputPathType,
    cifti_out: str,
    override_mapping_check: bool = False,
    fisher_z: bool = False,
) -> CiftiPairwiseCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_a: first input cifti file.
        cifti_b: second input cifti file.
        cifti_out: output cifti file.
        override_mapping_check: don't check the mappings for compatibility,\
            only check length.
        fisher_z: apply fisher small z transform (ie, artanh) to correlation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-pairwise-correlation",
        "cifti-a": cifti_a,
        "cifti-b": cifti_b,
        "cifti-out": cifti_out,
        "override-mapping-check": override_mapping_check,
        "fisher-z": fisher_z,
    }
    return params


def cifti_pairwise_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiPairwiseCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-a", None) is None:
        raise StyxValidationError("`cifti-a` must not be None")
    if not isinstance(params["cifti-a"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-a` has the wrong type: Received `{type(params.get("cifti-a", None))}` expected `InputPathType`')
    if params.get("cifti-b", None) is None:
        raise StyxValidationError("`cifti-b` must not be None")
    if not isinstance(params["cifti-b"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-b` has the wrong type: Received `{type(params.get("cifti-b", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("override-mapping-check", False) is None:
        raise StyxValidationError("`override-mapping-check` must not be None")
    if not isinstance(params["override-mapping-check"], bool):
        raise StyxValidationError(f'`override-mapping-check` has the wrong type: Received `{type(params.get("override-mapping-check", False))}` expected `bool`')
    if params.get("fisher-z", False) is None:
        raise StyxValidationError("`fisher-z` must not be None")
    if not isinstance(params["fisher-z"], bool):
        raise StyxValidationError(f'`fisher-z` has the wrong type: Received `{type(params.get("fisher-z", False))}` expected `bool`')


def cifti_pairwise_correlation_cargs(
    params: CiftiPairwiseCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-pairwise-correlation"
    ])
    cargs.append(execution.input_file(params.get("cifti-a", None)))
    cargs.append(execution.input_file(params.get("cifti-b", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("override-mapping-check", False):
        cargs.append("-override-mapping-check")
    if params.get("fisher-z", False):
        cargs.append("-fisher-z")
    return cargs


def cifti_pairwise_correlation_outputs(
    params: CiftiPairwiseCorrelationParamsDict,
    execution: Execution,
) -> CiftiPairwiseCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiPairwiseCorrelationOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_pairwise_correlation_execute(
    params: CiftiPairwiseCorrelationParamsDict,
    runner: Runner | None = None,
) -> CiftiPairwiseCorrelationOutputs:
    """
    CORRELATE PAIRED ROWS BETWEEN TWO CIFTI FILES.
    
    For each row in <cifti-a>, correlate it with the same row in <cifti-b>, and
    put the result in the same row of <cifti-out>, which has only one column.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiPairwiseCorrelationOutputs`).
    """
    cifti_pairwise_correlation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_PAIRWISE_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = cifti_pairwise_correlation_cargs(params, execution)
    ret = cifti_pairwise_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_pairwise_correlation(
    cifti_a: InputPathType,
    cifti_b: InputPathType,
    cifti_out: str,
    override_mapping_check: bool = False,
    fisher_z: bool = False,
    runner: Runner | None = None,
) -> CiftiPairwiseCorrelationOutputs:
    """
    CORRELATE PAIRED ROWS BETWEEN TWO CIFTI FILES.
    
    For each row in <cifti-a>, correlate it with the same row in <cifti-b>, and
    put the result in the same row of <cifti-out>, which has only one column.
    
    Args:
        cifti_a: first input cifti file.
        cifti_b: second input cifti file.
        cifti_out: output cifti file.
        override_mapping_check: don't check the mappings for compatibility,\
            only check length.
        fisher_z: apply fisher small z transform (ie, artanh) to correlation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiPairwiseCorrelationOutputs`).
    """
    params = cifti_pairwise_correlation_params(
        cifti_a=cifti_a,
        cifti_b=cifti_b,
        cifti_out=cifti_out,
        override_mapping_check=override_mapping_check,
        fisher_z=fisher_z,
    )
    return cifti_pairwise_correlation_execute(params, runner)


__all__ = [
    "CIFTI_PAIRWISE_CORRELATION_METADATA",
    "CiftiPairwiseCorrelationOutputs",
    "CiftiPairwiseCorrelationParamsDict",
    "CiftiPairwiseCorrelationParamsDictTagged",
    "cifti_pairwise_correlation",
    "cifti_pairwise_correlation_execute",
    "cifti_pairwise_correlation_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_AVERAGE_ROI_CORRELATION_METADATA = Metadata(
    id="8675ac82eeff102d67fcb3c8cd17fa1826495162.workbench",
    name="cifti-average-roi-correlation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiAverageRoiCorrelationCiftiRoiParamsDictNoTag = typing.TypedDict('_CiftiAverageRoiCorrelationCiftiRoiParamsDictNoTag', {
    "roi-cifti": InputPathType,
    "in-memory": bool,
})
CiftiAverageRoiCorrelationCiftiRoiParamsDictTagged = typing.TypedDict('CiftiAverageRoiCorrelationCiftiRoiParamsDictTagged', {
    "@type": typing.Literal["cifti-roi"],
    "roi-cifti": InputPathType,
    "in-memory": bool,
})
CiftiAverageRoiCorrelationCiftiRoiParamsDict = _CiftiAverageRoiCorrelationCiftiRoiParamsDictNoTag | CiftiAverageRoiCorrelationCiftiRoiParamsDictTagged


_CiftiAverageRoiCorrelationCiftiParamsDictNoTag = typing.TypedDict('_CiftiAverageRoiCorrelationCiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
})
CiftiAverageRoiCorrelationCiftiParamsDictTagged = typing.TypedDict('CiftiAverageRoiCorrelationCiftiParamsDictTagged', {
    "@type": typing.Literal["cifti"],
    "cifti-in": InputPathType,
})
CiftiAverageRoiCorrelationCiftiParamsDict = _CiftiAverageRoiCorrelationCiftiParamsDictNoTag | CiftiAverageRoiCorrelationCiftiParamsDictTagged


_CiftiAverageRoiCorrelationParamsDictNoTag = typing.TypedDict('_CiftiAverageRoiCorrelationParamsDictNoTag', {
    "cifti-out": str,
    "cifti-roi": typing.NotRequired[CiftiAverageRoiCorrelationCiftiRoiParamsDict | None],
    "cifti": typing.NotRequired[list[CiftiAverageRoiCorrelationCiftiParamsDict] | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiAverageRoiCorrelationParamsDictTagged = typing.TypedDict('CiftiAverageRoiCorrelationParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-average-roi-correlation"],
    "cifti-out": str,
    "cifti-roi": typing.NotRequired[CiftiAverageRoiCorrelationCiftiRoiParamsDict | None],
    "cifti": typing.NotRequired[list[CiftiAverageRoiCorrelationCiftiParamsDict] | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiAverageRoiCorrelationParamsDict = _CiftiAverageRoiCorrelationParamsDictNoTag | CiftiAverageRoiCorrelationParamsDictTagged


def cifti_average_roi_correlation_cifti_roi(
    roi_cifti: InputPathType,
    in_memory: bool = False,
) -> CiftiAverageRoiCorrelationCiftiRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_cifti: the roi cifti file.
        in_memory: cache the roi in memory so that it isn't re-read for each\
            input cifti.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti-roi",
        "roi-cifti": roi_cifti,
        "in-memory": in_memory,
    }
    return params


def cifti_average_roi_correlation_cifti_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageRoiCorrelationCiftiRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-cifti", None) is None:
        raise StyxValidationError("`roi-cifti` must not be None")
    if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType`')
    if params.get("in-memory", False) is None:
        raise StyxValidationError("`in-memory` must not be None")
    if not isinstance(params["in-memory"], bool):
        raise StyxValidationError(f'`in-memory` has the wrong type: Received `{type(params.get("in-memory", False))}` expected `bool`')


def cifti_average_roi_correlation_cifti_roi_cargs(
    params: CiftiAverageRoiCorrelationCiftiRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti-roi",
        execution.input_file(params.get("roi-cifti", None))
    ])
    if params.get("in-memory", False):
        cargs.append("-in-memory")
    return cargs


def cifti_average_roi_correlation_cifti(
    cifti_in: InputPathType,
) -> CiftiAverageRoiCorrelationCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: a cifti file to average across.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti-in": cifti_in,
    }
    return params


def cifti_average_roi_correlation_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageRoiCorrelationCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')


def cifti_average_roi_correlation_cifti_cargs(
    params: CiftiAverageRoiCorrelationCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti",
        execution.input_file(params.get("cifti-in", None))
    ])
    return cargs


class CiftiAverageRoiCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiAverageRoiCorrelationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_average_roi_correlation_params(
    cifti_out: str,
    cifti_roi: CiftiAverageRoiCorrelationCiftiRoiParamsDict | None = None,
    cifti: list[CiftiAverageRoiCorrelationCiftiParamsDict] | None = None,
    cerebellum_surf: InputPathType | None = None,
    right_surf: InputPathType | None = None,
    left_surf: InputPathType | None = None,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
) -> CiftiAverageRoiCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: output cifti file.
        cifti_roi: cifti file containing combined weights.
        cifti: specify an input cifti file.
        cerebellum_surf: specify the cerebellum surface for vertex area\
            correction\
            \
            the cerebellum surface file.
        right_surf: specify the right surface for vertex area correction\
            \
            the right surface file.
        left_surf: specify the left surface for vertex area correction\
            \
            the left surface file.
        roi_vol: voxel weights to use\
            \
            the roi volume file.
        roi_metric: weights to use for cerebellum surface\
            \
            the cerebellum roi as a metric file.
        roi_metric_: weights to use for right hempsphere\
            \
            the right roi as a metric file.
        roi_metric_2: weights to use for left hempsphere\
            \
            the left roi as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-average-roi-correlation",
        "cifti-out": cifti_out,
    }
    if cifti_roi is not None:
        params["cifti-roi"] = cifti_roi
    if cifti is not None:
        params["cifti"] = cifti
    if cerebellum_surf is not None:
        params["cerebellum-surf"] = cerebellum_surf
    if right_surf is not None:
        params["right-surf"] = right_surf
    if left_surf is not None:
        params["left-surf"] = left_surf
    if roi_vol is not None:
        params["roi-vol"] = roi_vol
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    if roi_metric_ is not None:
        params["roi-metric"] = roi_metric_
    if roi_metric_2 is not None:
        params["roi-metric"] = roi_metric_2
    return params


def cifti_average_roi_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageRoiCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("cifti-roi", None) is not None:
        cifti_average_roi_correlation_cifti_roi_validate(params["cifti-roi"])
    if params.get("cifti", None) is not None:
        if not isinstance(params["cifti"], list):
            raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `list[CiftiAverageRoiCorrelationCiftiParamsDict] | None`')
        for e in params["cifti"]:
            cifti_average_roi_correlation_cifti_validate(e)
    if params.get("cerebellum-surf", None) is not None:
        if not isinstance(params["cerebellum-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`cerebellum-surf` has the wrong type: Received `{type(params.get("cerebellum-surf", None))}` expected `InputPathType | None`')
    if params.get("right-surf", None) is not None:
        if not isinstance(params["right-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`right-surf` has the wrong type: Received `{type(params.get("right-surf", None))}` expected `InputPathType | None`')
    if params.get("left-surf", None) is not None:
        if not isinstance(params["left-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`left-surf` has the wrong type: Received `{type(params.get("left-surf", None))}` expected `InputPathType | None`')
    if params.get("roi-vol", None) is not None:
        if not isinstance(params["roi-vol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_average_roi_correlation_cargs(
    params: CiftiAverageRoiCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-average-roi-correlation"
    ])
    cargs.append(params.get("cifti-out", None))
    if params.get("cifti-roi", None) is not None or params.get("cifti", None) is not None:
        cargs.extend([
            *(cifti_average_roi_correlation_cifti_roi_cargs(params.get("cifti-roi", None), execution) if (params.get("cifti-roi", None) is not None) else []),
            *([a for c in [cifti_average_roi_correlation_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c] if (params.get("cifti", None) is not None) else [])
        ])
    if params.get("cerebellum-surf", None) is not None:
        cargs.extend([
            "-cerebellum-area-surf",
            execution.input_file(params.get("cerebellum-surf", None))
        ])
    if params.get("right-surf", None) is not None:
        cargs.extend([
            "-right-area-surf",
            execution.input_file(params.get("right-surf", None))
        ])
    if params.get("left-surf", None) is not None:
        cargs.extend([
            "-left-area-surf",
            execution.input_file(params.get("left-surf", None))
        ])
    if params.get("roi-vol", None) is not None:
        cargs.extend([
            "-vol-roi",
            execution.input_file(params.get("roi-vol", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-cerebellum-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-right-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-left-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_average_roi_correlation_outputs(
    params: CiftiAverageRoiCorrelationParamsDict,
    execution: Execution,
) -> CiftiAverageRoiCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiAverageRoiCorrelationOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_average_roi_correlation_execute(
    params: CiftiAverageRoiCorrelationParamsDict,
    runner: Runner | None = None,
) -> CiftiAverageRoiCorrelationOutputs:
    """
    CORRELATE ROI AVERAGE WITH ALL ROWS THEN AVERAGE ACROSS SUBJECTS.
    
    Averages rows for each map of the ROI(s), takes the correlation of each ROI
    average to the rest of the rows in the same file, applies the fisher small z
    transform, then averages the results across all files. ROIs are always
    treated as weighting functions, including negative values. For efficiency,
    ensure that everything that is not intended to be used is zero in the ROI
    map. If -cifti-roi is specified, -left-roi, -right-roi, -cerebellum-roi, and
    -vol-roi must not be specified. If multiple non-cifti ROI files are
    specified, they must have the same number of columns.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageRoiCorrelationOutputs`).
    """
    cifti_average_roi_correlation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_AVERAGE_ROI_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = cifti_average_roi_correlation_cargs(params, execution)
    ret = cifti_average_roi_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_average_roi_correlation(
    cifti_out: str,
    cifti_roi: CiftiAverageRoiCorrelationCiftiRoiParamsDict | None = None,
    cifti: list[CiftiAverageRoiCorrelationCiftiParamsDict] | None = None,
    cerebellum_surf: InputPathType | None = None,
    right_surf: InputPathType | None = None,
    left_surf: InputPathType | None = None,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
    runner: Runner | None = None,
) -> CiftiAverageRoiCorrelationOutputs:
    """
    CORRELATE ROI AVERAGE WITH ALL ROWS THEN AVERAGE ACROSS SUBJECTS.
    
    Averages rows for each map of the ROI(s), takes the correlation of each ROI
    average to the rest of the rows in the same file, applies the fisher small z
    transform, then averages the results across all files. ROIs are always
    treated as weighting functions, including negative values. For efficiency,
    ensure that everything that is not intended to be used is zero in the ROI
    map. If -cifti-roi is specified, -left-roi, -right-roi, -cerebellum-roi, and
    -vol-roi must not be specified. If multiple non-cifti ROI files are
    specified, they must have the same number of columns.
    
    Args:
        cifti_out: output cifti file.
        cifti_roi: cifti file containing combined weights.
        cifti: specify an input cifti file.
        cerebellum_surf: specify the cerebellum surface for vertex area\
            correction\
            \
            the cerebellum surface file.
        right_surf: specify the right surface for vertex area correction\
            \
            the right surface file.
        left_surf: specify the left surface for vertex area correction\
            \
            the left surface file.
        roi_vol: voxel weights to use\
            \
            the roi volume file.
        roi_metric: weights to use for cerebellum surface\
            \
            the cerebellum roi as a metric file.
        roi_metric_: weights to use for right hempsphere\
            \
            the right roi as a metric file.
        roi_metric_2: weights to use for left hempsphere\
            \
            the left roi as a metric file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageRoiCorrelationOutputs`).
    """
    params = cifti_average_roi_correlation_params(
        cifti_out=cifti_out,
        cifti_roi=cifti_roi,
        cifti=cifti,
        cerebellum_surf=cerebellum_surf,
        right_surf=right_surf,
        left_surf=left_surf,
        roi_vol=roi_vol,
        roi_metric=roi_metric,
        roi_metric_=roi_metric_,
        roi_metric_2=roi_metric_2,
    )
    return cifti_average_roi_correlation_execute(params, runner)


__all__ = [
    "CIFTI_AVERAGE_ROI_CORRELATION_METADATA",
    "CiftiAverageRoiCorrelationCiftiParamsDict",
    "CiftiAverageRoiCorrelationCiftiParamsDictTagged",
    "CiftiAverageRoiCorrelationCiftiRoiParamsDict",
    "CiftiAverageRoiCorrelationCiftiRoiParamsDictTagged",
    "CiftiAverageRoiCorrelationOutputs",
    "CiftiAverageRoiCorrelationParamsDict",
    "CiftiAverageRoiCorrelationParamsDictTagged",
    "cifti_average_roi_correlation",
    "cifti_average_roi_correlation_cifti",
    "cifti_average_roi_correlation_cifti_roi",
    "cifti_average_roi_correlation_execute",
    "cifti_average_roi_correlation_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_ALL_LABELS_TO_ROIS_METADATA = Metadata(
    id="25c70d1f4cda55f2df56316907bd5e6bb29642d1.workbench",
    name="volume-all-labels-to-rois",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeAllLabelsToRoisParamsDictNoTag = typing.TypedDict('_VolumeAllLabelsToRoisParamsDictNoTag', {
    "label-in": InputPathType,
    "map": str,
    "volume-out": str,
})
VolumeAllLabelsToRoisParamsDictTagged = typing.TypedDict('VolumeAllLabelsToRoisParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-all-labels-to-rois"],
    "label-in": InputPathType,
    "map": str,
    "volume-out": str,
})
VolumeAllLabelsToRoisParamsDict = _VolumeAllLabelsToRoisParamsDictNoTag | VolumeAllLabelsToRoisParamsDictTagged


class VolumeAllLabelsToRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeAllLabelsToRoisParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_all_labels_to_rois_params(
    label_in: InputPathType,
    map_: str,
    volume_out: str,
) -> VolumeAllLabelsToRoisParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input volume label file.
        map_: the number or name of the label map to use.
        volume_out: the output volume file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-all-labels-to-rois",
        "label-in": label_in,
        "map": map_,
        "volume-out": volume_out,
    }
    return params


def volume_all_labels_to_rois_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeAllLabelsToRoisParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("map", None) is None:
        raise StyxValidationError("`map` must not be None")
    if not isinstance(params["map"], str):
        raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')


def volume_all_labels_to_rois_cargs(
    params: VolumeAllLabelsToRoisParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-all-labels-to-rois"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("map", None))
    cargs.append(params.get("volume-out", None))
    return cargs


def volume_all_labels_to_rois_outputs(
    params: VolumeAllLabelsToRoisParamsDict,
    execution: Execution,
) -> VolumeAllLabelsToRoisOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeAllLabelsToRoisOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_all_labels_to_rois_execute(
    params: VolumeAllLabelsToRoisParamsDict,
    runner: Runner | None = None,
) -> VolumeAllLabelsToRoisOutputs:
    """
    MAKE ROIS FROM ALL LABELS IN A VOLUME FRAME.
    
    The output volume has a frame for each label in the specified input frame,
    other than the ??? label, each of which contains an ROI of all voxels that
    are set to the corresponding label.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeAllLabelsToRoisOutputs`).
    """
    volume_all_labels_to_rois_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_ALL_LABELS_TO_ROIS_METADATA)
    params = execution.params(params)
    cargs = volume_all_labels_to_rois_cargs(params, execution)
    ret = volume_all_labels_to_rois_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_all_labels_to_rois(
    label_in: InputPathType,
    map_: str,
    volume_out: str,
    runner: Runner | None = None,
) -> VolumeAllLabelsToRoisOutputs:
    """
    MAKE ROIS FROM ALL LABELS IN A VOLUME FRAME.
    
    The output volume has a frame for each label in the specified input frame,
    other than the ??? label, each of which contains an ROI of all voxels that
    are set to the corresponding label.
    
    Args:
        label_in: the input volume label file.
        map_: the number or name of the label map to use.
        volume_out: the output volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeAllLabelsToRoisOutputs`).
    """
    params = volume_all_labels_to_rois_params(
        label_in=label_in,
        map_=map_,
        volume_out=volume_out,
    )
    return volume_all_labels_to_rois_execute(params, runner)


__all__ = [
    "VOLUME_ALL_LABELS_TO_ROIS_METADATA",
    "VolumeAllLabelsToRoisOutputs",
    "VolumeAllLabelsToRoisParamsDict",
    "VolumeAllLabelsToRoisParamsDictTagged",
    "volume_all_labels_to_rois",
    "volume_all_labels_to_rois_execute",
    "volume_all_labels_to_rois_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_RESAMPLE_METADATA = Metadata(
    id="c5578cc16cfdc8f6e4a75e75e6c10f719016d2e0.workbench",
    name="label-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelResampleAreaSurfsParamsDictNoTag = typing.TypedDict('_LabelResampleAreaSurfsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
LabelResampleAreaSurfsParamsDictTagged = typing.TypedDict('LabelResampleAreaSurfsParamsDictTagged', {
    "@type": typing.Literal["area-surfs"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
LabelResampleAreaSurfsParamsDict = _LabelResampleAreaSurfsParamsDictNoTag | LabelResampleAreaSurfsParamsDictTagged


_LabelResampleAreaMetricsParamsDictNoTag = typing.TypedDict('_LabelResampleAreaMetricsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
LabelResampleAreaMetricsParamsDictTagged = typing.TypedDict('LabelResampleAreaMetricsParamsDictTagged', {
    "@type": typing.Literal["area-metrics"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
LabelResampleAreaMetricsParamsDict = _LabelResampleAreaMetricsParamsDictNoTag | LabelResampleAreaMetricsParamsDictTagged


_LabelResampleValidRoiOutParamsDictNoTag = typing.TypedDict('_LabelResampleValidRoiOutParamsDictNoTag', {
    "roi-out": str,
})
LabelResampleValidRoiOutParamsDictTagged = typing.TypedDict('LabelResampleValidRoiOutParamsDictTagged', {
    "@type": typing.Literal["valid-roi-out"],
    "roi-out": str,
})
LabelResampleValidRoiOutParamsDict = _LabelResampleValidRoiOutParamsDictNoTag | LabelResampleValidRoiOutParamsDictTagged


_LabelResampleParamsDictNoTag = typing.TypedDict('_LabelResampleParamsDictNoTag', {
    "label-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "method": str,
    "label-out": str,
    "area-surfs": typing.NotRequired[LabelResampleAreaSurfsParamsDict | None],
    "area-metrics": typing.NotRequired[LabelResampleAreaMetricsParamsDict | None],
    "valid-roi-out": typing.NotRequired[LabelResampleValidRoiOutParamsDict | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "bypass-sphere-check": bool,
    "largest": bool,
})
LabelResampleParamsDictTagged = typing.TypedDict('LabelResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/label-resample"],
    "label-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "method": str,
    "label-out": str,
    "area-surfs": typing.NotRequired[LabelResampleAreaSurfsParamsDict | None],
    "area-metrics": typing.NotRequired[LabelResampleAreaMetricsParamsDict | None],
    "valid-roi-out": typing.NotRequired[LabelResampleValidRoiOutParamsDict | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "bypass-sphere-check": bool,
    "largest": bool,
})
LabelResampleParamsDict = _LabelResampleParamsDictNoTag | LabelResampleParamsDictTagged


def label_resample_area_surfs(
    current_area: InputPathType,
    new_area: InputPathType,
) -> LabelResampleAreaSurfsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant anatomical surface with <current-sphere> mesh.
        new_area: a relevant anatomical surface with <new-sphere> mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "area-surfs",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def label_resample_area_surfs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelResampleAreaSurfsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def label_resample_area_surfs_cargs(
    params: LabelResampleAreaSurfsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-area-surfs",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def label_resample_area_metrics(
    current_area: InputPathType,
    new_area: InputPathType,
) -> LabelResampleAreaMetricsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for <current-sphere> mesh.
        new_area: a metric file with vertex areas for <new-sphere> mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "area-metrics",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def label_resample_area_metrics_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelResampleAreaMetricsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def label_resample_area_metrics_cargs(
    params: LabelResampleAreaMetricsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-area-metrics",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


class LabelResampleValidRoiOutOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelResampleValidRoiOutParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_out: OutputPathType
    """the output roi as a metric"""


def label_resample_valid_roi_out(
    roi_out: str,
) -> LabelResampleValidRoiOutParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_out: the output roi as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "valid-roi-out",
        "roi-out": roi_out,
    }
    return params


def label_resample_valid_roi_out_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelResampleValidRoiOutParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-out", None) is None:
        raise StyxValidationError("`roi-out` must not be None")
    if not isinstance(params["roi-out"], str):
        raise StyxValidationError(f'`roi-out` has the wrong type: Received `{type(params.get("roi-out", None))}` expected `str`')


def label_resample_valid_roi_out_cargs(
    params: LabelResampleValidRoiOutParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-valid-roi-out",
        params.get("roi-out", None)
    ])
    return cargs


def label_resample_valid_roi_out_outputs(
    params: LabelResampleValidRoiOutParamsDict,
    execution: Execution,
) -> LabelResampleValidRoiOutOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelResampleValidRoiOutOutputs(
        root=execution.output_file("."),
        roi_out=execution.output_file(params.get("roi-out", None)),
    )
    return ret


class LabelResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label file"""
    valid_roi_out: LabelResampleValidRoiOutOutputs | None
    """Outputs from `label_resample_valid_roi_out_outputs`."""


def label_resample_params(
    label_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    method: str,
    label_out: str,
    area_surfs: LabelResampleAreaSurfsParamsDict | None = None,
    area_metrics: LabelResampleAreaMetricsParamsDict | None = None,
    valid_roi_out: LabelResampleValidRoiOutParamsDict | None = None,
    roi_metric: InputPathType | None = None,
    bypass_sphere_check: bool = False,
    largest: bool = False,
) -> LabelResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the label file to resample.
        current_sphere: a sphere surface with the mesh that the label file is\
            currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        method: the method name.
        label_out: the output label file.
        area_surfs: specify surfaces to do vertex area correction based on.
        area_metrics: specify vertex area metrics to do area correction based\
            on.
        valid_roi_out: output the ROI of vertices that got data from valid\
            source vertices.
        roi_metric: use an input roi on the current mesh to exclude non-data\
            vertices\
            \
            the roi, as a metric file.
        bypass_sphere_check: ADVANCED: allow the current and new 'spheres' to\
            have arbitrary shape as long as they follow the same contour.
        largest: use only the label of the vertex with the largest weight.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-resample",
        "label-in": label_in,
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
        "method": method,
        "label-out": label_out,
        "bypass-sphere-check": bypass_sphere_check,
        "largest": largest,
    }
    if area_surfs is not None:
        params["area-surfs"] = area_surfs
    if area_metrics is not None:
        params["area-metrics"] = area_metrics
    if valid_roi_out is not None:
        params["valid-roi-out"] = valid_roi_out
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def label_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("method", None) is None:
        raise StyxValidationError("`method` must not be None")
    if not isinstance(params["method"], str):
        raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("area-surfs", None) is not None:
        label_resample_area_surfs_validate(params["area-surfs"])
    if params.get("area-metrics", None) is not None:
        label_resample_area_metrics_validate(params["area-metrics"])
    if params.get("valid-roi-out", None) is not None:
        label_resample_valid_roi_out_validate(params["valid-roi-out"])
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("bypass-sphere-check", False) is None:
        raise StyxValidationError("`bypass-sphere-check` must not be None")
    if not isinstance(params["bypass-sphere-check"], bool):
        raise StyxValidationError(f'`bypass-sphere-check` has the wrong type: Received `{type(params.get("bypass-sphere-check", False))}` expected `bool`')
    if params.get("largest", False) is None:
        raise StyxValidationError("`largest` must not be None")
    if not isinstance(params["largest"], bool):
        raise StyxValidationError(f'`largest` has the wrong type: Received `{type(params.get("largest", False))}` expected `bool`')


def label_resample_cargs(
    params: LabelResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-resample"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(execution.input_file(params.get("current-sphere", None)))
    cargs.append(execution.input_file(params.get("new-sphere", None)))
    cargs.append(params.get("method", None))
    cargs.append(params.get("label-out", None))
    if params.get("area-surfs", None) is not None or params.get("area-metrics", None) is not None or params.get("valid-roi-out", None) is not None:
        cargs.extend([
            *(label_resample_area_surfs_cargs(params.get("area-surfs", None), execution) if (params.get("area-surfs", None) is not None) else []),
            *(label_resample_area_metrics_cargs(params.get("area-metrics", None), execution) if (params.get("area-metrics", None) is not None) else []),
            *(label_resample_valid_roi_out_cargs(params.get("valid-roi-out", None), execution) if (params.get("valid-roi-out", None) is not None) else [])
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-current-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("bypass-sphere-check", False):
        cargs.append("-bypass-sphere-check")
    if params.get("largest", False):
        cargs.append("-largest")
    return cargs


def label_resample_outputs(
    params: LabelResampleParamsDict,
    execution: Execution,
) -> LabelResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelResampleOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
        valid_roi_out=label_resample_valid_roi_out_outputs(params.get("valid-roi-out"), execution) if params.get("valid-roi-out") else None,
    )
    return ret


def label_resample_execute(
    params: LabelResampleParamsDict,
    runner: Runner | None = None,
) -> LabelResampleOutputs:
    """
    RESAMPLE A LABEL FILE TO A DIFFERENT MESH.
    
    Resamples a label file, given two spherical surfaces that are in register.
    If ADAP_BARY_AREA is used, exactly one of -area-surfs or -area-metrics must
    be specified.
    
    The ADAP_BARY_AREA method is recommended for label data, because it should
    be better at resolving vertices that are near multiple labels, or in case of
    downsampling. Midthickness surfaces are recommended for the vertex areas for
    most data.
    
    The -largest option results in nearest vertex behavior when used with
    BARYCENTRIC, as it uses the value of the source vertex that has the largest
    weight.
    
    When -largest is not specified, the vertex weights are summed according to
    which label they correspond to, and the label with the largest sum is used.
    
    The <method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelResampleOutputs`).
    """
    label_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = label_resample_cargs(params, execution)
    ret = label_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_resample(
    label_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    method: str,
    label_out: str,
    area_surfs: LabelResampleAreaSurfsParamsDict | None = None,
    area_metrics: LabelResampleAreaMetricsParamsDict | None = None,
    valid_roi_out: LabelResampleValidRoiOutParamsDict | None = None,
    roi_metric: InputPathType | None = None,
    bypass_sphere_check: bool = False,
    largest: bool = False,
    runner: Runner | None = None,
) -> LabelResampleOutputs:
    """
    RESAMPLE A LABEL FILE TO A DIFFERENT MESH.
    
    Resamples a label file, given two spherical surfaces that are in register.
    If ADAP_BARY_AREA is used, exactly one of -area-surfs or -area-metrics must
    be specified.
    
    The ADAP_BARY_AREA method is recommended for label data, because it should
    be better at resolving vertices that are near multiple labels, or in case of
    downsampling. Midthickness surfaces are recommended for the vertex areas for
    most data.
    
    The -largest option results in nearest vertex behavior when used with
    BARYCENTRIC, as it uses the value of the source vertex that has the largest
    weight.
    
    When -largest is not specified, the vertex weights are summed according to
    which label they correspond to, and the label with the largest sum is used.
    
    The <method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Args:
        label_in: the label file to resample.
        current_sphere: a sphere surface with the mesh that the label file is\
            currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        method: the method name.
        label_out: the output label file.
        area_surfs: specify surfaces to do vertex area correction based on.
        area_metrics: specify vertex area metrics to do area correction based\
            on.
        valid_roi_out: output the ROI of vertices that got data from valid\
            source vertices.
        roi_metric: use an input roi on the current mesh to exclude non-data\
            vertices\
            \
            the roi, as a metric file.
        bypass_sphere_check: ADVANCED: allow the current and new 'spheres' to\
            have arbitrary shape as long as they follow the same contour.
        largest: use only the label of the vertex with the largest weight.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelResampleOutputs`).
    """
    params = label_resample_params(
        label_in=label_in,
        current_sphere=current_sphere,
        new_sphere=new_sphere,
        method=method,
        label_out=label_out,
        area_surfs=area_surfs,
        area_metrics=area_metrics,
        valid_roi_out=valid_roi_out,
        roi_metric=roi_metric,
        bypass_sphere_check=bypass_sphere_check,
        largest=largest,
    )
    return label_resample_execute(params, runner)


__all__ = [
    "LABEL_RESAMPLE_METADATA",
    "LabelResampleAreaMetricsParamsDict",
    "LabelResampleAreaMetricsParamsDictTagged",
    "LabelResampleAreaSurfsParamsDict",
    "LabelResampleAreaSurfsParamsDictTagged",
    "LabelResampleOutputs",
    "LabelResampleParamsDict",
    "LabelResampleParamsDictTagged",
    "LabelResampleValidRoiOutOutputs",
    "LabelResampleValidRoiOutParamsDict",
    "LabelResampleValidRoiOutParamsDictTagged",
    "label_resample",
    "label_resample_area_metrics",
    "label_resample_area_surfs",
    "label_resample_execute",
    "label_resample_params",
    "label_resample_valid_roi_out",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SCENE_FILE_RELOCATE_METADATA = Metadata(
    id="cce9293384c531d31f6ab6adfc95c967b2740fd1.workbench",
    name="scene-file-relocate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SceneFileRelocateParamsDictNoTag = typing.TypedDict('_SceneFileRelocateParamsDictNoTag', {
    "input-scene": str,
    "output-scene": str,
})
SceneFileRelocateParamsDictTagged = typing.TypedDict('SceneFileRelocateParamsDictTagged', {
    "@type": typing.Literal["workbench/scene-file-relocate"],
    "input-scene": str,
    "output-scene": str,
})
SceneFileRelocateParamsDict = _SceneFileRelocateParamsDictNoTag | SceneFileRelocateParamsDictTagged


class SceneFileRelocateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SceneFileRelocateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_file_relocate_params(
    input_scene: str,
    output_scene: str,
) -> SceneFileRelocateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_scene: the scene file to use.
        output_scene: output - the new scene file to create.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/scene-file-relocate",
        "input-scene": input_scene,
        "output-scene": output_scene,
    }
    return params


def scene_file_relocate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneFileRelocateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input-scene", None) is None:
        raise StyxValidationError("`input-scene` must not be None")
    if not isinstance(params["input-scene"], str):
        raise StyxValidationError(f'`input-scene` has the wrong type: Received `{type(params.get("input-scene", None))}` expected `str`')
    if params.get("output-scene", None) is None:
        raise StyxValidationError("`output-scene` must not be None")
    if not isinstance(params["output-scene"], str):
        raise StyxValidationError(f'`output-scene` has the wrong type: Received `{type(params.get("output-scene", None))}` expected `str`')


def scene_file_relocate_cargs(
    params: SceneFileRelocateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-scene-file-relocate"
    ])
    cargs.append(params.get("input-scene", None))
    cargs.append(params.get("output-scene", None))
    return cargs


def scene_file_relocate_outputs(
    params: SceneFileRelocateParamsDict,
    execution: Execution,
) -> SceneFileRelocateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SceneFileRelocateOutputs(
        root=execution.output_file("."),
    )
    return ret


def scene_file_relocate_execute(
    params: SceneFileRelocateParamsDict,
    runner: Runner | None = None,
) -> SceneFileRelocateOutputs:
    """
    RECREATE SCENE FILE IN NEW LOCATION.
    
    Scene files contain internal relative paths, such that moving or copying a
    scene file will cause it to lose track of the files it refers to. This
    command makes a modified copy of the scene file, changing the relative paths
    to refer to the new relative locations of the files.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileRelocateOutputs`).
    """
    scene_file_relocate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_FILE_RELOCATE_METADATA)
    params = execution.params(params)
    cargs = scene_file_relocate_cargs(params, execution)
    ret = scene_file_relocate_outputs(params, execution)
    execution.run(cargs)
    return ret


def scene_file_relocate(
    input_scene: str,
    output_scene: str,
    runner: Runner | None = None,
) -> SceneFileRelocateOutputs:
    """
    RECREATE SCENE FILE IN NEW LOCATION.
    
    Scene files contain internal relative paths, such that moving or copying a
    scene file will cause it to lose track of the files it refers to. This
    command makes a modified copy of the scene file, changing the relative paths
    to refer to the new relative locations of the files.
    
    Args:
        input_scene: the scene file to use.
        output_scene: output - the new scene file to create.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileRelocateOutputs`).
    """
    params = scene_file_relocate_params(
        input_scene=input_scene,
        output_scene=output_scene,
    )
    return scene_file_relocate_execute(params, runner)


__all__ = [
    "SCENE_FILE_RELOCATE_METADATA",
    "SceneFileRelocateOutputs",
    "SceneFileRelocateParamsDict",
    "SceneFileRelocateParamsDictTagged",
    "scene_file_relocate",
    "scene_file_relocate_execute",
    "scene_file_relocate_params",
]
